/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DrlPackageAdder.java 4499 2009-07-23 12:20:54Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.io.Reader;
import java.io.StringReader;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.drools.RuleBase;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;
import org.drools.rule.Rule;
import org.osgi.framework.Bundle;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * 
 * 
 */
public class DrlPackageAdder extends AbstractPackageAdder
							 implements IPackageAdder {
	
	// -------------------------------------------------------------------------
	// Variables
	// -------------------------------------------------------------------------
 
	/**
     * Logger.
     */
    private static Logger logger =
        Logger.getLogger(DrlPackageAdder.class.getName());

    /**
     * The Drools package name
     */
    private final String packageName;
    
    /**
     * The Drl file contents
     */
    private final String drlFile;
    
    
	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------
    
    /**
     * Create a new package adder to add rules belonging to a given bundle
     * into a working memory.
     * 
     * @param bundle the bundle containing rules
     * @param globals the globals
     * @param the targeted working memory name
     */
    public DrlPackageAdder(final Bundle bundle,
    					   Map<String, Object> globals,
    					   final String packageName,
    					   final String drlFile) {

        super(bundle, globals);
        this.packageName = packageName;
        this.drlFile = drlFile;
    }

    /**
     * Create a new package adder to add rules belonging to a given bundle
     * into the default working memory.
     * 
     * @param bundle the bundle containing rules
     * @param globals the globals
     */
    public DrlPackageAdder(final Bundle bundle,
    					   Map<String, Object> globals,
    					   final String workingMemoryName,
    					   final String packageName,
    					   final String drlFile) {
    	
    	super(bundle, globals, workingMemoryName);
    	this.packageName = packageName;
        this.drlFile = drlFile;
    }
    
    
	// -------------------------------------------------------------------------
	// Getters & Setters
	// -------------------------------------------------------------------------    
    
    /**
     * Get the Drools package name
     */
    public String getPackageName() {
    	return this.packageName;
    }
    
    
	// -------------------------------------------------------------------------
	// Implementation of IPackageAdder
	// -------------------------------------------------------------------------
    
    /**
     * Add a rules package into the providing {@link RuleBase}, by using the
     * given {@link PackageBuilder}.
     * 
     * @param packageBuilder a package builder
     * @param ruleBase a rule base
     * @throws Exception if a package cannot be built or added to the rule base
     */
    public void addPackage(final PackageBuilder builder, final RuleBase ruleBase)
    throws Exception {

	    Reader drlFileReader = new StringReader(drlFile);
        try {
            builder.addPackageFromDrl(drlFileReader);
            logger.log(Level.INFO, "Drl file loaded");
        } finally {
        	drlFileReader.close();
        }

        Package pack = builder.getPackage();
        if(pack != null) {
            pack.checkValidity();
            Rule[] rules = pack.getRules();
            if(rules.length != 0) {
                ruleBase.addPackage(pack);
            }
        }
    }
    
}
